<?php
class I18N {
    /**
     * Path of gettext directory
     * @access private
     * @var    string
     */
	var $_sTranslationsPath = "locale";

    /**
     * Last language loaded
     * @access private
     * @var    string
     */
	var $_sLocale = "en-US";

    /**
     * Last domain loaded
     * @access private
     * @var    string
     */
	var $_sDefaultDomain = "main";

    /**
     * Array of domains loaded
     * @access private
     * @var    array
     */
	var $_aDomains = array();
	
	var $_aAvailableLocale;

	
	/**
	* @var boolean
	*/
	var $gettext_loaded = false;

	var $show_errormessages = false;

	/**
	* @param string $domains
	* @param string $TranslationsPath
	* @return void
	*/
	function I18N($domain = 'main', $TranslationsPath = NULL ) {
		global $Config_root_dir,$Config_locale;
		$this->_sLocale = $this->isValidLocaleCode($Config_locale) ? $Config_locale : 'en_US';
		$this->_sTranslationsPath = is_null($TranslationsPath) ? $Config_root_dir.'/locale' : $TranslationsPath;
		
		if( is_file($this->_sTranslationsPath . DIRECTORY_SEPARATOR . $this->_sLocale . DIRECTORY_SEPARATOR . 'LC_MESSAGES' . DIRECTORY_SEPARATOR . $domain . '.mo') ){
			$this->_sDefaultDomain = $domain;
			$this->setGettext();
		}else {}
	} // end constructor

	function setDomains($domain , $TranslationsPath) {
		if( false === array_search($domain,$this->_aDomains) ){
			array_push($this->_aDomains,$domain);
			return bindtextdomain($domain , $TranslationsPath);
		}
		return false;
	} // end function

	/**
	* loads the gettext data
	* @return boolean
	*/
	function setGettext() {
		if ($this->gettext_loaded === true) return true;

		if (!function_exists('gettext')) {
			if ($this->show_errormessages)
				echo '<br />Gettext extention not installed!<br />';
			return false;
		} // end if
		
		//setlocale(LC_ALL, ''); // Enable this if you have problems with gettext. Maybe it helps
		putenv( 'LANG=' . $this->_sLocale );
		setlocale( LC_ALL , $this->_sLocale );
		
		bindtextdomain($this->_sDefaultDomain, $this->_sTranslationsPath);
		textdomain($this->_sDefaultDomain);
		bind_textdomain_codeset($this->_sDefaultDomain, 'UTF-8');
		
		$this->gettext_loaded = true;
		return true;
	} // end function
	
	/**
	 * Retrieves the translation of $translationstring. If the gettext isn't loaded the original text is returned.
	 * @param string $translationstring Text to translate.
	 * @param string $domain Domain to retrieve the translated text.
	 * @return string Translated text or error message
	 */
	function translate($translationstring, $domain = null) {
		if ($this->gettext_loaded === false) return $translationstring;
		
		if ($domain) {
			$translated = dgettext( $domain , $translationstring );
		}else{
			$translated = gettext( $translationstring );
		}
		if ($this->show_errormessages && $translated === $translationstring) {
			trigger_error('TRANSLATION ERROR: String "' . $translationstring . '" not found in translation table '. $domain, E_USER_WARNING);
		} // end if
       return $translated;
	} // end function

	/**
	 * Retrieve the plural or single form based on the amount. If the 'gettext' isn't loaded, then a comparison will be made
	 * and either $plural or $single parameters returned.
	 *
	 * If the domain does exist, then the parameters will be passed to the 'dngettext'
	 *
	 * @param string $singular The text that will be used if $number is 1
	 * @param string $plural The text that will be used if $number is not 1
	 * @param int $number The number to compare against to use either $single or $plural
	 * @param string $domain Optional. The domain identifier the text should be retrieved in
	 * @return string Either $single or $plural translated text
	 */
	function translate_plural($singular, $plural, $number, $domain = null) {
		if ($this->gettext_loaded === false) return 1 == $number? $singular : $plural;
		
		if ($domain) {
			$translated = dngettext( $domain , $singular , $plural , $number );
		}else{
			$translated = ngettext( $singular , $plural , $number );
		}
		return $translated;
	}

	/**
	* shortcut wrapper method for the translate() method
	*/
	function _($translationstring, $domain = null) {
		return $this->translate($translationstring, $domain);
	} // end function
	
	/**
	 * Displays the returned translated text from translate().
	 *
	 * @see translate() Echoes returned translate() string
	 * @param string $translationstring Text to translate
	 * @param string $domain Optional. Domain to retrieve the translated text
	 */
	function _e( $translationstring, $domain = null ) {
		echo $this->translate( $translationstring, $domain );
	}

	/**
	 * Retrieves the translation of $translationstring and escapes it for safe use in HTML.
	 * If there is no translation, or the domain isn't loaded the original text is returned.
	 * Converts a number of special characters into their HTML entities.
	 *
	 * Specifically deals with: &, <, >, ", and '.
	 *
	 * $quote_style can be set to ENT_COMPAT to encode " to &quot;
	 * , or ENT_QUOTES to do both. Default is ENT_NOQUOTES where no quotes are encoded.
	 *
	 * @param string $translationstring Text to translate
	 * @param string $domain Optional. Domain to retrieve the translated text
	 * @param mixed $quote_style Optional. Converts double quotes if set to ENT_COMPAT, both single and double if set to ENT_QUOTES or none if set to ENT_NOQUOTES.
	 * @param string $charset Optional. The character encoding of the string. Default is UTF-8.
	 * @return string Translated text
	 */
	function esc( $translationstring, $domain = null, $quote_style = ENT_QUOTES, $charset = 'UTF-8' ) {
		return htmlspecialchars( $this->translate($translationstring, $domain), $quote_style, $charset );
	}
	
	function esc_e( $translationstring, $domain = null ) {
		echo $this->esc( $translationstring, $domain );
	}
	
	/**
	* shortcut wrapper method for the translate_plural() method
	*/
	function _n( $single, $plural, $number, $domain = null ) {
		return $this->translate_plural( $single, $plural, $number, $domain );
	}
	
	/**
	* checks if a given locale is valid
	* @param string $code de, de-AT,...
	* @return boolean
	*/
	function isValidLocaleCode($code = '') {
		return preg_match('(^([a-zA-Z]{2})((_|-)[a-zA-Z]{2})?$)', $code);
	} // end function

	/**
	 * Get all available languages based on the presence of *.mo files in a given directory. The default directory is LANG DIR.
	 *
	 * @param string $dir A directory in which to search for language files. The default directory is LANG DIR.
	 * @return array Array of language codes or an empty array if no languages are present.  Language codes are formed by stripping the .mo extension from the language file names.
	 */
	function get_available_languages( $dir = null ) {
		$languages = array();
	
		foreach( (array)glob( ( is_null( $dir) ? $this->_sTranslationsPath : $dir ) . '/*.mo' ) as $lang_file ) {
			$lang_file = basename($lang_file, '.mo');
			$languages[] = $lang_file;
		}
	
		return $languages;
	}

} // end class I18N

class I18NUser {
	
	/**
	* retreive the available translator locales
	* @return boolean
	*/
	function loadAvailableLanguage() {
		$this->_aAvailableLocale = array();
		$root = $this->_sTranslationsPath . '/' ;
		$handle = @opendir($root);
		while ($lang_dir = @readdir($handle)) {
			if (!is_dir($root . $lang_dir) || !$this->isValidLocaleCode($lang_dir) ) {
				continue;
			} // end if
			$this->_aAvailableLocale[] = $lang_dir;
		} // end while
		@closedir($handle);
		unset($root, $handle);
		return count($this->_aAvailableLocale);
	} // end function

	/**
	* returns a country from a given domainname
	* @param string $host
	* @return mixed
	*/
	function domain2Country($host = '') {
		if (($lastdot = strrpos($host,'.')) < 1) {
			return (boolean) FALSE;
		} // end if
		$parts = explode('.', $host);
		$domain_name = array_pop($parts);
		/* top level domains (com, org, gov, aero, info)
			or eu-domain are all english */
		if (strlen($domain_name) > 2 || $domain_name === 'eu') {
			return 'en';
		} elseif (strlen($domain_name) == 2) { // country domains
			return $domain_name;
		} // end if
		return (boolean) FALSE;
	} // end function

	/**
	* translates the users ip to a domainname
	* @return boolean
	*/
	function readUserIP() {
		$country = $this->domain2Country(gethostbyaddr($_SERVER['REMOTE_ADDR']));

		if ($country == FALSE) {
			$country = $this->domain2Country($_SERVER['SERVER_NAME']);
		} // end if

		if ($country == FALSE) {
			return (boolean) FALSE;
		} // end if

		$this->countries[] = $country;
		$this->countries = array_unique($this->countries);
		return (boolean) TRUE;
	} // end function
	
	/**
	* gets the data from the http header request
	* @return boolean
	*/
	function readUserHeader() {
		$client_header = explode(',', $_SERVER['HTTP_ACCEPT_LANGUAGE']);

		if (count($client_header) < 1) {
			return (boolean) FALSE;
		} // end if

		foreach ($client_header as $raw_entry) {
			$temp = explode(';', $raw_entry);
			$locale = trim($temp[0]);
			$this->addLocale($locale);
		} // end foreach

		if (is_array($this->languages)) {
			$this->languages = array_unique($this->languages);
		} // end if

		if (is_array($this->countries)) {
			$this->countries = array_unique($this->countries);
		} // end if
		return (boolean) TRUE;
	} // end function
	
}
?>